@@ -90,6 +90,8 @@
 extern HMODULE _priv_gst_dll_handle;
 #endif
 
+#include "glib-compat-private.h"
+
 #define GST_CAT_DEFAULT preset_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
@@ -546,7 +548,7 @@ gst_preset_default_get_preset_names (GstPreset * preset)
   }
 
   /* sort the array now */
-  g_qsort_with_data (groups, num_groups, sizeof (gchar *),
+  g_sort_array (groups, num_groups, sizeof (gchar *),
       (GCompareDataFunc) compare_strings, NULL);
 
   return groups;
@@ -1054,7 +1056,7 @@ no_presets:
  *
  * Get a copy of preset names as a %NULL terminated string array.
  *
- * Returns: (transfer full) (array zero-terminated=1) (element-type gchar*):
+ * Returns: (transfer full) (array zero-terminated=1) (element-type utf8):
  *     list with names, use g_strfreev() after usage.
  */
 gchar **
@@ -1071,7 +1073,7 @@ gst_preset_get_preset_names (GstPreset * preset)
  *
  * Get a the names of the GObject properties that can be used for presets.
  *
- * Returns: (transfer full) (array zero-terminated=1) (element-type gchar*): an
+ * Returns: (transfer full) (array zero-terminated=1) (element-type utf8): an
  *   array of property names which should be freed with g_strfreev() after use.
  */
 gchar **
