@@ -21,6 +21,7 @@
 
 /**
  * SECTION:gsttoc
+ * @title: GstToc
  * @short_description: Generic table of contents support
  * @see_also: #GstStructure, #GstEvent, #GstMessage, #GstQuery
  *
@@ -82,7 +83,6 @@
 #include "gstvalue.h"
 #include "gsttoc.h"
 #include "gstpad.h"
-#include "gstquark.h"
 
 struct _GstTocEntry
 {
@@ -139,7 +139,7 @@ gst_toc_new (GstTocScope scope)
   g_return_val_if_fail (scope == GST_TOC_SCOPE_GLOBAL ||
       scope == GST_TOC_SCOPE_CURRENT, NULL);
 
-  toc = g_slice_new0 (GstToc);
+  toc = g_new0 (GstToc, 1);
 
   gst_mini_object_init (GST_MINI_OBJECT_CAST (toc), 0, GST_TYPE_TOC,
       (GstMiniObjectCopyFunction) gst_toc_copy, NULL,
@@ -168,7 +168,7 @@ gst_toc_get_scope (const GstToc * toc)
 /**
  * gst_toc_set_tags:
  * @toc: A #GstToc instance
- * @tags: (allow-none) (transfer full): A #GstTagList or %NULL
+ * @tags: (nullable) (transfer full): A #GstTagList or %NULL
  *
  * Set a #GstTagList with tags for the complete @toc.
  */
@@ -186,7 +186,7 @@ gst_toc_set_tags (GstToc * toc, GstTagList * tags)
 /**
  * gst_toc_merge_tags:
  * @toc: A #GstToc instance
- * @tags: (allow-none): A #GstTagList or %NULL
+ * @tags: (nullable): A #GstTagList or %NULL
  * @mode: A #GstTagMergeMode
  *
  * Merge @tags into the existing tags of @toc using @mode.
@@ -212,7 +212,7 @@ gst_toc_merge_tags (GstToc * toc, GstTagList * tags, GstTagMergeMode mode)
  *
  * Gets the tags for @toc.
  *
- * Returns: (transfer none): A #GstTagList for @entry
+ * Returns: (transfer none) (nullable): A #GstTagList for @entry
  */
 GstTagList *
 gst_toc_get_tags (const GstToc * toc)
@@ -268,7 +268,7 @@ gst_toc_entry_new_internal (GstTocEntryType type, const gchar * uid)
 {
   GstTocEntry *entry;
 
-  entry = g_slice_new0 (GstTocEntry);
+  entry = g_new0 (GstTocEntry, 1);
 
   gst_mini_object_init (GST_MINI_OBJECT_CAST (entry), 0, GST_TYPE_TOC_ENTRY,
       (GstMiniObjectCopyFunction) gst_toc_entry_copy, NULL,
@@ -289,7 +289,7 @@ gst_toc_entry_new_internal (GstTocEntryType type, const gchar * uid)
  *
  * Create new #GstTocEntry structure.
  *
- * Returns: newly allocated #GstTocEntry structure, free it with gst_toc_entry_unref().
+ * Returns: (transfer full): newly allocated #GstTocEntry structure, free it with gst_toc_entry_unref().
  */
 GstTocEntry *
 gst_toc_entry_new (GstTocEntryType type, const gchar * uid)
@@ -308,7 +308,11 @@ gst_toc_free (GstToc * toc)
   if (toc->tags != NULL)
     gst_tag_list_unref (toc->tags);
 
-  g_slice_free (GstToc, toc);
+#ifdef USE_POISONING
+  memset (toc, 0xff, sizeof (GstToc));
+#endif
+
+  g_free (toc);
 }
 
 static void
@@ -324,7 +328,11 @@ gst_toc_entry_free (GstTocEntry * entry)
   if (entry->tags != NULL)
     gst_tag_list_unref (entry->tags);
 
-  g_slice_free (GstTocEntry, entry);
+#ifdef USE_POISONING
+  memset (entry, 0xff, sizeof (GstTocEntry));
+#endif
+
+  g_free (entry);
 }
 
 static GstTocEntry *
@@ -406,8 +414,7 @@ gst_toc_deep_copy_toc_entries (GList * entry_list)
  *
  * Copy #GstTocEntry with all subentries (deep copy).
  *
- * Returns: (nullable): newly allocated #GstTocEntry in case of
- * success, %NULL otherwise; free it when done with
+ * Returns: (transfer full): newly allocated #GstTocEntry; free it when done with
  * gst_toc_entry_unref().
  */
 static GstTocEntry *
@@ -441,8 +448,7 @@ gst_toc_entry_copy (const GstTocEntry * entry)
  *
  * Copy #GstToc with all subentries (deep copy).
  *
- * Returns: (nullable): newly allocated #GstToc in case of success,
- * %NULL otherwise; free it when done with gst_toc_unref().
+ * Returns: (transfer full): newly allocated #GstToc; free it when done with gst_toc_unref().
  */
 static GstToc *
 gst_toc_copy (const GstToc * toc)
@@ -486,9 +492,9 @@ gst_toc_entry_set_start_stop_times (GstTocEntry * entry, gint64 start,
 /**
  * gst_toc_entry_get_start_stop_times:
  * @entry: #GstTocEntry to get values from.
- * @start: (out) (allow-none): the storage for the start value, leave
+ * @start: (out) (optional): the storage for the start value, leave
  *   %NULL if not need.
- * @stop: (out) (allow-none): the storage for the stop value, leave
+ * @stop: (out) (optional): the storage for the stop value, leave
  *   %NULL if not need.
  *
  * Get @start and @stop values from the @entry and write them into appropriate
@@ -534,9 +540,9 @@ gst_toc_entry_set_loop (GstTocEntry * entry, GstTocLoopType loop_type,
 /**
  * gst_toc_entry_get_loop:
  * @entry: #GstTocEntry to get values from.
- * @loop_type: (out) (allow-none): the storage for the loop_type
+ * @loop_type: (out) (optional): the storage for the loop_type
  *             value, leave %NULL if not need.
- * @repeat_count: (out) (allow-none): the storage for the repeat_count
+ * @repeat_count: (out) (optional): the storage for the repeat_count
  *                value, leave %NULL if not need.
  *
  * Get @loop_type and @repeat_count values from the @entry and write them into
@@ -699,7 +705,7 @@ gst_toc_entry_get_sub_entries (const GstTocEntry * entry)
 /**
  * gst_toc_entry_set_tags:
  * @entry: A #GstTocEntry instance
- * @tags: (allow-none) (transfer full): A #GstTagList or %NULL
+ * @tags: (nullable) (transfer full): A #GstTagList or %NULL
  *
  * Set a #GstTagList with tags for the complete @entry.
  */
@@ -717,7 +723,7 @@ gst_toc_entry_set_tags (GstTocEntry * entry, GstTagList * tags)
 /**
  * gst_toc_entry_merge_tags:
  * @entry: A #GstTocEntry instance
- * @tags: (allow-none): A #GstTagList or %NULL
+ * @tags: (nullable): A #GstTagList or %NULL
  * @mode: A #GstTagMergeMode
  *
  * Merge @tags into the existing tags of @entry using @mode.
@@ -744,7 +750,7 @@ gst_toc_entry_merge_tags (GstTocEntry * entry, GstTagList * tags,
  *
  * Gets the tags for @entry.
  *
- * Returns: (transfer none): A #GstTagList for @entry
+ * Returns: (nullable) (transfer none): A #GstTagList for @entry
  */
 GstTagList *
 gst_toc_entry_get_tags (const GstTocEntry * entry)
@@ -760,7 +766,7 @@ gst_toc_entry_get_tags (const GstTocEntry * entry)
  *
  * Gets the parent #GstToc of @entry.
  *
- * Returns: (transfer none): The parent #GstToc of @entry
+ * Returns: (transfer none) (nullable): The parent #GstToc of @entry
  */
 GstToc *
 gst_toc_entry_get_toc (GstTocEntry * entry)
@@ -776,7 +782,7 @@ gst_toc_entry_get_toc (GstTocEntry * entry)
  *
  * Gets the parent #GstTocEntry of @entry.
  *
- * Returns: (transfer none): The parent #GstTocEntry of @entry
+ * Returns: (transfer none) (nullable): The parent #GstTocEntry of @entry
  */
 GstTocEntry *
 gst_toc_entry_get_parent (GstTocEntry * entry)
