@@ -23,6 +23,7 @@
 
 /**
  * SECTION:gstinterpolationcontrolsource
+ * @title: GstInterpolationControlSource
  * @short_description: interpolation control source
  *
  * #GstInterpolationControlSource is a #GstControlSource, that interpolates values between user-given
@@ -35,6 +36,9 @@
  * All functions are MT-safe.
  *
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <glib-object.h>
 #include <gst/gst.h>
@@ -168,7 +172,7 @@ interpolate_none_get_value_array (GstTimedValueControlSource * self,
 
 
 /*  linear interpolation */
-/*  smoothes inbetween values */
+/*  smoothes in between values */
 static inline gdouble
 _interpolate_linear (GstClockTime timestamp1, gdouble value1,
     GstClockTime timestamp2, gdouble value2, GstClockTime timestamp)
@@ -437,7 +441,7 @@ interpolate_cubic_get_value_array (GstTimedValueControlSource * self,
  */
 
 static void
-_interpolate_cubic_mono_update_cache (GstTimedValueControlSource * self)
+_interpolate_cubic_monotonic_update_cache (GstTimedValueControlSource * self)
 {
   gint i, n = self->nvalues;
   gdouble *dxs = g_new0 (gdouble, n);
@@ -498,9 +502,9 @@ _interpolate_cubic_mono_update_cache (GstTimedValueControlSource * self)
     inv_dx = 1.0 / dxs[i];
     common = c1 + c1s[i + 1] - m - m;
 
-    cp->cache.cubic_mono.c1s = c1;
-    cp->cache.cubic_mono.c2s = (m - c1 - common) * inv_dx;
-    cp->cache.cubic_mono.c3s = common * inv_dx * inv_dx;
+    cp->cache.cubic_monotonic.c1s = c1;
+    cp->cache.cubic_monotonic.c2s = (m - c1 - common) * inv_dx;
+    cp->cache.cubic_monotonic.c3s = common * inv_dx * inv_dx;
 
     iter = g_sequence_iter_next (iter);
   }
@@ -513,12 +517,12 @@ _interpolate_cubic_mono_update_cache (GstTimedValueControlSource * self)
 }
 
 static inline gdouble
-_interpolate_cubic_mono (GstTimedValueControlSource * self,
+_interpolate_cubic_monotonic (GstTimedValueControlSource * self,
     GstControlPoint * cp1, gdouble value1, GstControlPoint * cp2,
     gdouble value2, GstClockTime timestamp)
 {
   if (!self->valid_cache) {
-    _interpolate_cubic_mono_update_cache (self);
+    _interpolate_cubic_monotonic_update_cache (self);
     self->valid_cache = TRUE;
   }
 
@@ -527,9 +531,9 @@ _interpolate_cubic_mono (GstTimedValueControlSource * self,
     gdouble diff2 = diff * diff;
     gdouble out;
 
-    out = value1 + cp1->cache.cubic_mono.c1s * diff;
-    out += cp1->cache.cubic_mono.c2s * diff2;
-    out += cp1->cache.cubic_mono.c3s * diff * diff2;
+    out = value1 + cp1->cache.cubic_monotonic.c1s * diff;
+    out += cp1->cache.cubic_monotonic.c2s * diff2;
+    out += cp1->cache.cubic_monotonic.c3s * diff * diff2;
     return out;
   } else {
     return value1;
@@ -537,7 +541,7 @@ _interpolate_cubic_mono (GstTimedValueControlSource * self,
 }
 
 static gboolean
-interpolate_cubic_mono_get (GstTimedValueControlSource * self,
+interpolate_cubic_monotonic_get (GstTimedValueControlSource * self,
     GstClockTime timestamp, gdouble * value)
 {
   gboolean ret = FALSE;
@@ -549,7 +553,7 @@ interpolate_cubic_mono_get (GstTimedValueControlSource * self,
   g_mutex_lock (&self->lock);
 
   if (_get_nearest_control_points (self, timestamp, &cp1, &cp2)) {
-    *value = _interpolate_cubic_mono (self, cp1, cp1->value, cp2,
+    *value = _interpolate_cubic_monotonic (self, cp1, cp1->value, cp2,
         (cp2 ? cp2->value : 0.0), timestamp);
     ret = TRUE;
   }
@@ -558,7 +562,7 @@ interpolate_cubic_mono_get (GstTimedValueControlSource * self,
 }
 
 static gboolean
-interpolate_cubic_mono_get_value_array (GstTimedValueControlSource * self,
+interpolate_cubic_monotonic_get_value_array (GstTimedValueControlSource * self,
     GstClockTime timestamp, GstClockTime interval, guint n_values,
     gdouble * values)
 {
@@ -581,7 +585,7 @@ interpolate_cubic_mono_get_value_array (GstTimedValueControlSource * self,
       _get_nearest_control_points2 (self, ts, &cp1, &cp2, &next_ts);
     }
     if (cp1) {
-      *values = _interpolate_cubic_mono (self, cp1, cp1->value, cp2,
+      *values = _interpolate_cubic_monotonic (self, cp1, cp1->value, cp2,
           (cp2 ? cp2->value : 0.0), ts);
       ret = TRUE;
       GST_LOG ("values[%3d]=%lf", i, *values);
@@ -603,15 +607,16 @@ static struct
   GstControlSourceGetValueArray get_value_array;
 } interpolation_modes[] = {
   {
-  (GstControlSourceGetValue) interpolate_none_get,
-        (GstControlSourceGetValueArray) interpolate_none_get_value_array}, {
-  (GstControlSourceGetValue) interpolate_linear_get,
-        (GstControlSourceGetValueArray) interpolate_linear_get_value_array}, {
-  (GstControlSourceGetValue) interpolate_cubic_get,
-        (GstControlSourceGetValueArray) interpolate_cubic_get_value_array}, {
-    (GstControlSourceGetValue) interpolate_cubic_mono_get,
+        (GstControlSourceGetValue) interpolate_none_get,
+      (GstControlSourceGetValueArray) interpolate_none_get_value_array}, {
+        (GstControlSourceGetValue) interpolate_linear_get,
+      (GstControlSourceGetValueArray) interpolate_linear_get_value_array}, {
+        (GstControlSourceGetValue) interpolate_cubic_get,
+      (GstControlSourceGetValueArray) interpolate_cubic_get_value_array}, {
+        (GstControlSourceGetValue) interpolate_cubic_monotonic_get,
         (GstControlSourceGetValueArray)
-interpolate_cubic_mono_get_value_array}};
+      interpolate_cubic_monotonic_get_value_array}
+};
 
 static const guint num_interpolation_modes = G_N_ELEMENTS (interpolation_modes);
 
@@ -620,27 +625,10 @@ enum
   PROP_MODE = 1
 };
 
-GType
-gst_interpolation_mode_get_type (void)
+struct _GstInterpolationControlSourcePrivate
 {
-  static gsize gtype = 0;
-  static const GEnumValue values[] = {
-    {GST_INTERPOLATION_MODE_NONE, "GST_INTERPOLATION_MODE_NONE", "none"},
-    {GST_INTERPOLATION_MODE_LINEAR, "GST_INTERPOLATION_MODE_LINEAR", "linear"},
-    {GST_INTERPOLATION_MODE_CUBIC, "GST_INTERPOLATION_MODE_CUBIC", "cubic"},
-    {GST_INTERPOLATION_MODE_CUBIC_MONO, "GST_INTERPOLATION_MODE_CUBIC_MONO",
-        "cubic-mono"},
-    {0, NULL, NULL}
-  };
-
-  if (g_once_init_enter (&gtype)) {
-    GType tmp = g_enum_register_static ("GstInterpolationMode", values);
-    g_once_init_leave (&gtype, tmp);
-  }
-
-  return (GType) gtype;
-}
-
+  GstInterpolationMode interpolation_mode;
+};
 
 #define _do_init \
   GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "interpolation control source", 0, \
@@ -648,13 +636,9 @@ gst_interpolation_mode_get_type (void)
 
 G_DEFINE_TYPE_WITH_CODE (GstInterpolationControlSource,
     gst_interpolation_control_source, GST_TYPE_TIMED_VALUE_CONTROL_SOURCE,
+    G_ADD_PRIVATE (GstInterpolationControlSource)
     _do_init);
 
-struct _GstInterpolationControlSourcePrivate
-{
-  GstInterpolationMode interpolation_mode;
-};
-
 /**
  * gst_interpolation_control_source_new:
  *
@@ -665,7 +649,13 @@ struct _GstInterpolationControlSourcePrivate
 GstControlSource *
 gst_interpolation_control_source_new (void)
 {
-  return g_object_newv (GST_TYPE_INTERPOLATION_CONTROL_SOURCE, 0, NULL);
+  GstControlSource *csource =
+      g_object_new (GST_TYPE_INTERPOLATION_CONTROL_SOURCE, NULL);
+
+  /* Clear floating flag */
+  gst_object_ref_sink (csource);
+
+  return csource;
 }
 
 static gboolean
@@ -695,9 +685,7 @@ static gboolean
 static void
 gst_interpolation_control_source_init (GstInterpolationControlSource * self)
 {
-  self->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE (self, GST_TYPE_INTERPOLATION_CONTROL_SOURCE,
-      GstInterpolationControlSourcePrivate);
+  self->priv = gst_interpolation_control_source_get_instance_private (self);
   gst_interpolation_control_source_set_interpolation_mode (self,
       GST_INTERPOLATION_MODE_NONE);
 }
@@ -742,10 +730,6 @@ gst_interpolation_control_source_class_init (GstInterpolationControlSourceClass
     * klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  //GstControlSourceClass *csource_class = GST_CONTROL_SOURCE_CLASS (klass);
-
-  g_type_class_add_private (klass,
-      sizeof (GstInterpolationControlSourcePrivate));
 
   gobject_class->set_property = gst_interpolation_control_source_set_property;
   gobject_class->get_property = gst_interpolation_control_source_get_property;
