@@ -26,6 +26,7 @@
 
 #include <gst/gst.h>
 #include <stdio.h>
+#include <gst/base/gstqueuearray.h>
 
 G_BEGIN_DECLS
 
@@ -85,6 +86,7 @@ struct _GstQueue2
 
   /* Position of src/sink */
   GstClockTime sinktime, srctime;
+  GstClockTime sink_start_time;
   /* TRUE if either position needs to be recalculated */
   gboolean sink_tainted, src_tainted;
   /* Bitrates taken from tags */
@@ -98,17 +100,20 @@ struct _GstQueue2
   gboolean unexpected;
 
   /* the queue of data we're keeping our hands on */
-  GQueue queue;
+  GstVecDeque *queue;
 
   GCond query_handled;
   gboolean last_query; /* result of last serialized query */
+  GstQuery *last_handled_query;
 
   GstQueue2Size cur_level;       /* currently in the queue */
   GstQueue2Size max_level;       /* max. amount of data allowed in the queue */
   gboolean use_buffering;
   gboolean use_tags_bitrate;
+  gboolean use_bitrate_query;
   gboolean use_rate_estimate;
   GstClockTime buffering_interval;
+  guint downstream_bitrate;     /* the bitrate reported by downstream */
 
   /* low/high watermarks for buffering */
   gint low_watermark;
@@ -117,6 +122,7 @@ struct _GstQueue2
   /* current buffering state */
   gboolean is_buffering;
   gint buffering_percent;
+  gint last_posted_buffering_percent;
 
   /* for measuring input/output rates */
   GTimer *in_timer;
@@ -159,13 +165,12 @@ struct _GstQueue2
   guint64 ring_buffer_max_size;
   guint8 * ring_buffer;
 
-  volatile gint downstream_may_block;
+  gint downstream_may_block;
 
   GstBufferingMode mode;
   gint64 buffering_left;
   gint avg_in;
   gint avg_out;
-  gboolean percent_changed;
   GMutex buffering_post_lock; /* assures only one posted at a time */
 };
 
