@@ -19,11 +19,17 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include <gst/check/gstcheck.h>
 
@@ -68,6 +74,7 @@ cleanup_fdsrc (GstElement * fdsrc)
   gst_check_teardown_element (fdsrc);
 }
 
+#ifdef HAVE_PIPE
 GST_START_TEST (test_num_buffers)
 {
   GstElement *src;
@@ -87,9 +94,19 @@ GST_START_TEST (test_num_buffers)
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
 
+#if defined (G_OS_UNIX) && defined (O_NONBLOCK)
+  {
+    int flags;
+
+    flags = fcntl (pipe_fd[1], F_GETFL, 0);
+    fcntl (pipe_fd[1], F_SETFL, flags | O_NONBLOCK);
+  }
+#endif
+
   memset (data, 0, 4096);
   while (!have_eos) {
-    fail_if (write (pipe_fd[1], data, 4096) < 0);
+    int ret = write (pipe_fd[1], data, 4096);
+    fail_if (ret < 0 && errno != EAGAIN);
     g_usleep (100);
   }
 
@@ -149,6 +166,7 @@ GST_START_TEST (test_nonseeking)
 }
 
 GST_END_TEST;
+#endif /* HAVE_PIPE */
 
 GST_START_TEST (test_seeking)
 {
@@ -193,8 +211,10 @@ fdsrc_suite (void)
   TCase *tc_chain = tcase_create ("general");
 
   suite_add_tcase (s, tc_chain);
+#ifdef HAVE_PIPE
   tcase_add_test (tc_chain, test_num_buffers);
   tcase_add_test (tc_chain, test_nonseeking);
+#endif
   tcase_add_test (tc_chain, test_seeking);
 
   return s;
