@@ -18,6 +18,9 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <gst/check/gstcheck.h>
 
@@ -82,7 +85,7 @@ setup_output_pad (GstElement * element, GstStaticPadTemplate * tmpl)
       GINT_TO_POINTER (probe_id));
 
   /* request src pad */
-  srcpad = gst_element_get_request_pad (element, "src_%u");
+  srcpad = gst_element_request_pad_simple (element, "src_%u");
   fail_if (srcpad == NULL, "Could not get source pad from %s",
       GST_ELEMENT_NAME (element));
 
@@ -204,15 +207,7 @@ push_newsegment_events (GList * input_pads)
   GstSegment seg;
   GList *l;
 
-  seg.flags = GST_SEGMENT_FLAG_NONE;
-  seg.rate = seg.applied_rate = 1.0;
-  seg.format = GST_FORMAT_BYTES;
-  seg.base = 0;
-  seg.start = 0;
-  seg.stop = -1;
-  seg.time = 0;
-  seg.position = 0;
-  seg.duration = -1;
+  gst_segment_init (&seg, GST_FORMAT_BYTES);
 
   for (l = input_pads; l; l = l->next) {
     GstPad *pad = l->data;
@@ -300,7 +295,7 @@ setup_input_pad (GstElement * element)
   fail_if (input_pad == NULL, "Could not create a input_pad");
 
   /* request sink pad */
-  sinkpad = gst_element_get_request_pad (element, "sink_%u");
+  sinkpad = gst_element_request_pad_simple (element, "sink_%u");
   fail_if (sinkpad == NULL, "Could not get sink pad from %s",
       GST_ELEMENT_NAME (element));
 
@@ -497,10 +492,12 @@ input_selector_push_buffer (gint stream, enum InputSelectorResult res)
   }
 }
 
+/* Consumes a ref to the passed pad */
 static gpointer
 input_selector_do_push_eos (GstPad * pad)
 {
   gst_pad_push_event (pad, gst_event_new_eos ());
+  gst_object_unref (pad);
   return NULL;
 }
 
@@ -526,11 +523,11 @@ input_selector_push_eos (gint stream, gboolean active)
   if (active) {
     fail_unless (gst_pad_push_event (pad, gst_event_new_eos ()));
   } else {
-    /* The non-active pads will block when receving eos, so we need to do it
+    /* The non-active pads will block when receiving eos, so we need to do it
      * from a separate thread. This makes this test racy, but it should only
      * cause false positives, not false negatives */
     GThread *t = g_thread_new ("selector-test-push-eos",
-        (GThreadFunc) input_selector_do_push_eos, pad);
+        (GThreadFunc) input_selector_do_push_eos, gst_object_ref (pad));
 
     /* Sleep half a second to allow the other thread to execute, this is not
      * a definitive solution but there is no way to know when the
